@@ -3,6 +3,7 @@
 
 #include <Rcpp.h>
 #include <queue>
+#include <atomic>
 #include <functional>
 #include <memory>
 #include "timestamp.h"
@@ -56,7 +57,15 @@ public:
   StdFunctionCallback(Timestamp when, std::function<void (void)> func);
 
   void invoke() const {
+#ifdef RCPP_USING_UNWIND_PROTECT // See https://github.com/r-lib/later/issues/191
+    Rcpp::unwindProtect([this]() {
+      BEGIN_RCPP
+      func();
+      END_RCPP
+    });
+#else
     func();
+#endif
   }
 
   Rcpp::RObject rRepresentation() const;
@@ -68,7 +77,7 @@ private:
 
 class RcppFunctionCallback : public Callback {
 public:
-  RcppFunctionCallback(Timestamp when, Rcpp::Function func);
+  RcppFunctionCallback(Timestamp when, const Rcpp::Function& func);
 
   void invoke() const {
     func();
@@ -107,6 +116,7 @@ private:
   // objects to be copied on the wrong thread, and even trigger an R GC event
   // on the wrong thread. https://github.com/r-lib/later/issues/39
   cbSet queue;
+  std::atomic<int> fd_waits{};
   Mutex* mutex;
   ConditionVariable* condvar;
 
@@ -122,7 +132,7 @@ public:
 
   // Add a function to the registry, to be executed at `secs` seconds in
   // the future (i.e. relative to the current time).
-  uint64_t add(Rcpp::Function func, double secs);
+  uint64_t add(const Rcpp::Function& func, double secs);
 
   // Add a C function to the registry, to be executed at `secs` seconds in
   // the future (i.e. relative to the current time).
@@ -134,14 +144,14 @@ public:
   // Use this to determine the next time we need to pump events.
   Optional<Timestamp> nextTimestamp(bool recursive = true) const;
 
-  // Is the registry completely empty?
+  // Is the registry completely empty? (including later_fd waits)
   bool empty() const;
 
   // Is anything ready to execute?
   bool due(const Timestamp& time = Timestamp(), bool recursive = true) const;
 
-  // Pop and return an ordered list of functions to execute now.
-  std::vector<Callback_sp> take(size_t max = -1, const Timestamp& time = Timestamp());
+  // Pop and return a function to execute now.
+  Callback_sp pop(const Timestamp& time = Timestamp());
 
   // Wait until the next available callback is ready to execute.
   bool wait(double timeoutSecs, bool recursive) const;
@@ -149,6 +159,10 @@ public:
   // Return a List of items in the queue.
   Rcpp::List list() const;
 
+  // Increment and decrement the number of active later_fd waits
+  void fd_waits_incr();
+  void fd_waits_decr();
+
   // References to parent and children registries. These are used for
   // automatically running child loops. They should only be accessed and
   // modified from the main thread.
